@@ -29,11 +29,29 @@ def override_default_kwargs(method_to_wrap: typing.Callable) -> typing.Callable:
 class ModuleInput(abc.ABC):
     """Allocate an Module object as an input of another Module"""
 
-    def __init__(self, relevance: bool = True):
+    def __init__(
+        self,
+        relevance: bool = True,
+    ):
         self.relevance = relevance
 
+    def get_replication_key(
+        self,
+        parent: typing.Optional[Module] = None,
+        module_input_name: str = "",
+    ) -> str:
+        if parent:
+            return f"child.{str(parent)}.{module_input_name}"
+        else:
+            return ""
+
     @abc.abstractmethod
-    def __call__(self, module_container: ModuleContainer) -> Module:
+    def __call__(
+        self,
+        module_container: ModuleContainer,
+        parent: typing.Optional[Module] = None,
+        module_input_name: str = "",
+    ) -> Module:
         ...
 
 
@@ -42,7 +60,12 @@ class Catch(ModuleInput):
         super().__init__(*args, **kwargs)
         self.module_instance_name = module_instance_name
 
-    def __call__(self, module_container: ModuleContainer) -> Module:
+    def __call__(
+        self,
+        module_container: ModuleContainer,
+        parent: typing.Optional[Module] = None,
+        module_input_name: str = "",
+    ) -> Module:
         try:
             return module_container.get_module_by_name(self.module_instance_name)
         except (KeyError, InvalidModuleInstanceNameError):
@@ -66,8 +89,18 @@ class AutoSetup(ModuleInput):
         self.module_class = module_class
         self.module_kwargs = module_kwargs
 
-    def __call__(self, module_container: ModuleContainer) -> Module:
-        module = self.module_class(**self.module_kwargs)
+    def __call__(
+        self,
+        module_container: ModuleContainer,
+        parent: typing.Optional[Module] = None,
+        module_input_name: str = "",
+    ) -> Module:
+        module_kwargs = dict(self.module_kwargs)
+        if "replication_key" not in module_kwargs:
+            module_kwargs["replication_key"] = self.get_replication_key(
+                parent, module_input_name
+            )
+        module = self.module_class(**module_kwargs)
         module_instance_dict = {id(module): module}
         module_name = module.get_class_name()
         try:
@@ -86,6 +119,7 @@ class Module(
 ):
     def __init__(
         self,
+        replication_key: str = "",
         send_to_physical_output: bool = False,
         auto_stop: bool = True,
         fade_in_duration: float = 0.1,
@@ -93,6 +127,7 @@ class Module(
         module_input_dict: typing.Dict[str, ModuleInput] = dict([]),
         default_dict: typing.Dict[str, typing.Any] = dict([]),
     ):
+        self.replication_key = replication_key
         self.send_to_physical_output = send_to_physical_output
         self.auto_stop = auto_stop
         self.fade_in_duration = fade_in_duration
@@ -139,10 +174,11 @@ class Module(
 
             cls.__init__ = __init__
 
-    def __hash__(self) -> int:
-        # XXX: This method is overridden by dataclasses and
-        # has to be manually added again.
-        return walkman.NamedMixin.__hash__(self)
+    def __str__(self) -> str:
+        return f"{self.get_class_name()}.{self.replication_key}"
+
+    def __repr__(self) -> str:
+        return str(self)
 
     # ################## ABSTRACT PROPERTIES ################## #
 
@@ -152,8 +188,7 @@ class Module(
 
     # ################## PRIVATE METHODS     ################## #
 
-    def _play(self, duration: float = 0, delay: float = 0):
-        self.fader.play(dur=duration, delay=delay)
+    def _play_without_fader(self, duration: float = 0, delay: float = 0):
         self.pyo_object.play(dur=duration, delay=delay)
         for internal_pyo_object in self.internal_pyo_object_list:
             internal_pyo_object.play(dur=duration, delay=delay)
@@ -162,13 +197,19 @@ class Module(
             for channel_index, audio_stream in enumerate(self.pyo_object):
                 audio_stream.out(channel_index)
 
-    def _stop(self, wait: float = 0):
-        self.fader_stopper.play(delay=wait)
-        wait += self.fade_out_duration
+    def _play(self, duration: float = 0, delay: float = 0):
+        self.fader.play(dur=duration, delay=delay)
+        self._play_without_fader(duration, delay)
+
+    def _stop_without_fader(self, wait: float = 0):
         for internal_pyo_object in self.internal_pyo_object_list:
             internal_pyo_object.stop(wait=wait)
         self.pyo_object.stop(wait=wait)
 
+    def _stop(self, wait: float = 0):
+        self.fader_stopper.play(delay=wait)
+        self._stop_without_fader(wait=wait + self.fade_out_duration)
+
     def _initialise(self, **_):
         ...
 
@@ -194,7 +235,9 @@ class Module(
     def assign_module_inputs(self, module_container: ModuleContainer):
         if not self.has_assigned_inputs:
             for module_input_name, module_input in self.module_input_dict.items():
-                module = module_input(module_container)
+                module = module_input(
+                    module_container, parent=self, module_input_name=module_input_name
+                )
                 module.output_module_set.add(self)
                 setattr(self, module_input_name, module)
 
@@ -265,6 +308,12 @@ class Module(
         # Parse everything else to actual initialise method
         self._initialise(**new_kwargs)
 
+        # When switching playing cues the 'play' method
+        # won't be called. But we have to ensure that the envelopes
+        # are running once we start a new cue.
+        if self.is_playing:
+            self._play_without_fader()
+
         initialised_module_list.append(self)
         return tuple(initialised_module_list)
 
@@ -367,13 +416,34 @@ ReplicationKey = str
 ModuleNameToModuleClassDict = typing.Dict[str, typing.Type[Module]]
 
 
+class UndefinedModuleWarning(Warning):
+    def __init__(self, undefined_module_name: str):
+        super().__init__(
+            f"Found undefined module '{undefined_module_name}'. "
+            "WALKMAN ignored given module configurations."
+        )
+
+
+class NoPhysicalOutputWarning(Warning):
+    def __init__(self, module_without_output: Module):
+        output_chain = "\n- ".join([str(module) for module in module_without_output.module_output_chain])
+        super().__init__(
+            f"WALKMAN detected module '{str(module_without_output)}' which "
+            "has no connection to any other module which is"
+            " send to a physical output. This may be the result from "
+            f"bad routing. The given output chain is: \n\n- {output_chain}\n"
+        )
+
+
 class ModuleContainer(
     typing.Dict[ModuleName, typing.Dict[ReplicationKey, Module]], walkman.CloseMixin
 ):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        if not "empty" in self:
+
+        if "empty" not in self:
             self.update({"empty": {}})
+
         self["empty"].update(
             {
                 walkman.constants.EMPTY_MODULE_INSTANCE_NAME.split(".")[
@@ -384,6 +454,16 @@ class ModuleContainer(
         for module in self.module_tuple:
             self.prepare_module(module)
 
+        for module in self.module_tuple:
+            if not (is_send_to_physical_output := module.send_to_physical_output):
+                for output_module in module.module_output_chain:
+                    if (
+                        is_send_to_physical_output := output_module.send_to_physical_output
+                    ):
+                        break
+                if not is_send_to_physical_output:
+                    warnings.warn(NoPhysicalOutputWarning(module))
+
     def prepare_module(self, module_instance: Module):
         def assign_module_inputs(module_instance: Module):
             for module_input_name in module_instance.module_input_dict.keys():
@@ -438,6 +518,9 @@ class ModuleContainer(
                 cls.get_module_name_to_module_class_dict()
             )
 
+        # XXX: We have to iterate over 'module_name_to_module_class_dict'
+        # instead of iterating over module_name_to_replication_configuration_dict,
+        # because of syntactic sugar [configure.MODULE_NAME.REPLICATION_KEY.INPUT_NAME]
         module_name_to_module_container = {}
         for module_name, module_class in module_name_to_module_class_dict.items():
             try:
@@ -445,16 +528,24 @@ class ModuleContainer(
                     module_name_to_replication_configuration_dict[module_name]
                 )
             except KeyError:
-                replication_key_to_module_configuration_dict = {}
+                continue
+            # For debugging (see below)
+            del module_name_to_replication_configuration_dict[module_name]
             module_dict = {}
             for (
                 replication_key,
                 module_configuration,
             ) in replication_key_to_module_configuration_dict.items():
-                module = module_class(**module_configuration)
+                module = module_class(
+                    replication_key=replication_key, **module_configuration
+                )
                 module_dict[replication_key] = module
             if module_dict:
                 module_name_to_module_container.update({module_name: module_dict})
+
+        for module_name in module_name_to_replication_configuration_dict:
+            warnings.warn(UndefinedModuleWarning(module_name))
+
         return cls(module_name_to_module_container)
 
     def close(self):
