@@ -11,6 +11,7 @@ import walkman
 from . import base
 
 __all__ = (
+    "ModuleWithUneffectiveStop",
     "Empty",
     "Parameter",
     "AudioInput",
@@ -28,6 +29,14 @@ __all__ = (
 )
 
 
+class ModuleWithUneffectiveStop(base.Module):
+    def _stop(self, wait: float = 0):
+        ...
+
+    def stop(self, wait: float = 0) -> ModuleWithUneffectiveStop:
+        return self
+
+
 class Empty(base.Module):
     def _setup_pyo_object(self):
         super()._setup_pyo_object()
@@ -96,6 +105,7 @@ class Parameter(base.Module):
                 self.exponential_envelope,
             ]
         )
+        self.portamento = pyo.Port(self.selector)
 
         self.ENVELOPE_TYPE_TO_ENVELOPE = {
             "linear": self.linear_envelope,
@@ -108,6 +118,7 @@ class Parameter(base.Module):
                 self.linear_envelope,
                 self.exponential_envelope,
                 self.selector,
+                self.portamento,
             ]
         )
 
@@ -118,7 +129,7 @@ class Parameter(base.Module):
 
     @property
     def _pyo_object(self) -> pyo.PyoObject:
-        return self.selector
+        return self.portamento
 
     @property
     def duration(self) -> float:
@@ -128,7 +139,11 @@ class Parameter(base.Module):
         self,
         value: typing.Union[float, typing.List[typing.List[float]]] = 0,
         envelope_type: str = DEFAULT_ENVELOPE_TYPE,
+        rise_time: float = 0.001,
+        fall_time: float = 0.001,
     ):
+        self.portamento.setRiseTime(rise_time)
+        self.portamento.setFallTime(fall_time)
         selector_voice = None
         envelope_list = None
         if isinstance(value, list):
@@ -178,7 +193,7 @@ class AudioInput(ModuleWithDecibel):
         return self.denorm
 
 
-class MidiControlInput(base.Module):
+class MidiControlInput(ModuleWithUneffectiveStop):
     def __init__(self, midi_control_number: int = 0, midi_channel: int = 0, **kwargs):
         super().__init__(**kwargs)
         self.midi_control_number = midi_control_number
@@ -191,6 +206,10 @@ class MidiControlInput(base.Module):
         )
         self.internal_pyo_object_list.append(self.midi_input)
 
+    def _initialise(self, minima: float = 0, maxima: float = 1, **_):
+        self.midi_input.setMinScale(minima)
+        self.midi_input.setMaxScale(maxima)
+
     @property
     def _pyo_object(self) -> pyo.PyoObject:
         return self.midi_input
@@ -269,7 +288,7 @@ class Mixer(
 
         self.input_mixer = pyo.Mixer(outs=self.input_channel_count)
         self.output_mixer = pyo.Mixer(
-            outs=self.output_channel_count, mul=self.amplitude_signal_to
+            outs=self.output_channel_count  # , mul=self.amplitude_signal_to
         )
 
         for (
@@ -300,43 +319,45 @@ class Mixer(
     def _initialise(
         self, **audio_input_channel_mapping: typing.Dict[str, walkman.ChannelMapping]
     ):
-        # Input has the form: {"audio_input_0": {"channel_mapping": {0: 1}}}.
-        # This is better than {"audio_input_0_channel_mapping": ..}, because
-        # we don't have to process the keys and the UI in the toml file is
-        # more intuitive (
-        #       [mixer.0.audio_input_0.channel_mapping] instead of
-        #       [mixer.0.audio_input_0_channel_mapping]
-        # )
+        audio_input_key_to_channel_mapping_dict = {}
+        for (
+            audio_input_channel_mapping_key,
+            channel_mapping,
+        ) in audio_input_channel_mapping.items():
+            audio_input_key, *_ = audio_input_channel_mapping_key.split(
+                "_channel_mapping"
+            )
+            if audio_input_key in self.module_input_dict.keys():
+                audio_input_key_to_channel_mapping_dict.update(
+                    {audio_input_key: channel_mapping}
+                )
+            else:
+                warnings.warn(
+                    "Found invalid channel mapping "
+                    f"argument '{audio_input_channel_mapping_key}'!"
+                )
+
         for audio_input_key in self.module_input_dict.keys():
             module_instance = getattr(self, audio_input_key)
             if not isinstance(module_instance, Empty):
-                if audio_input_key not in audio_input_channel_mapping:
+                if audio_input_key not in audio_input_key_to_channel_mapping_dict:
                     channel_mapping = {}
                     for channel_index in range(len(module_instance.pyo_object)):
                         output_channel = channel_index % self.input_channel_count
                         channel_mapping.update({channel_index: output_channel})
-                    audio_input_channel_mapping.update(
-                        {audio_input_key: {"channel_mapping": channel_mapping}}
+                    audio_input_key_to_channel_mapping_dict.update(
+                        {audio_input_key: channel_mapping}
                     )
 
         for (
             audio_input_key,
-            audio_input_configuration,
-        ) in audio_input_channel_mapping.items():
-            try:
-                channel_mapping = audio_input_configuration["channel_mapping"]
-            except KeyError:
-                warnings.warn(
-                    "Found useless audio input configuration "
-                    f"'{audio_input_configuration}'. Only valid key"
-                    " is 'channel_mapping' for now."
+            channel_mapping,
+        ) in audio_input_key_to_channel_mapping_dict.items():
+            if isinstance(channel_mapping, (dict, walkman.ChannelMapping)):
+                channel_mapping = walkman.dict_or_channel_mapping_to_channel_mapping(
+                    channel_mapping
                 )
-                continue
-
-            channel_mapping = walkman.dict_or_channel_mapping_to_channel_mapping(
-                channel_mapping
-            )
-            self.activate_channel_mapping(audio_input_key, channel_mapping)
+                self.activate_channel_mapping(audio_input_key, channel_mapping)
 
     def _raise_illegal_audio_object_channel_warning(
         self,
@@ -550,9 +571,7 @@ class ButterworthHighpassFilter(
 class Equalizer(
     ModuleWithDecibel,
     audio_input=base.Catch(walkman.constants.EMPTY_MODULE_INSTANCE_NAME),
-    frequency=base.AutoSetup(
-        Parameter, module_kwargs={"default_dict": {"value": 500}}
-    ),
+    frequency=base.AutoSetup(Parameter, module_kwargs={"default_dict": {"value": 500}}),
     boost=base.AutoSetup(Parameter, module_kwargs={"default_dict": {"value": -3}}),
 ):
     FILTER_TYPE_TO_INTERNAL_FILTER_TYPE = {
